(* 
 * Js_of_ocaml examples
 * wavelet transform
 *)

module Html = Dom_html
let jstr = Js.string
let document = Html.window##document

(* Canvas 作成 *)  
let create_canvas () =
  let c = Html.createCanvas document in
  c##width <- 320;
  c##height <- 320;
  c
let canvas = create_canvas ()

let int_input name value =
  let res = document##createDocumentFragment() in
  Dom.appendChild res (document##createTextNode(jstr name));
  let input = Html.createInput ~_type:(jstr "text") document in
  input##value <- jstr (string_of_int !value);
  input##onchange <- Html.handler
    (fun _ ->
       begin try
         value := int_of_string (Js.to_string (input##value))
       with Invalid_argument _ ->
         ()
       end;
       input##value <- jstr (string_of_int !value);
       Js._false);
  Dom.appendChild res input;
  res

let button name callback =
  let res = document##createDocumentFragment() in
  let input = Html.createInput ~_type:(jstr "submit") document in
  input##value <- jstr name;
  input##onclick <- Html.handler callback;
  Dom.appendChild res input;
  res

let freq = ref 8
let tpi = 8. *. atan 1.
let h = 1. /. 1024.


let plot_array ctx t0 x0 x_arr =
  (* x_arr のプロット *)
  let plot n x =
    let nf = float_of_int n in
    if n = 0
    then ctx##moveTo (t0 +. nf *. 0.2, x0 -. x)
    else ctx##lineTo (t0 +. nf *. 0.2, x0 -. x)
  in

  let plot_xaxis n x =
    let nf = float_of_int n in
    if n = 0
    then ctx##moveTo (t0 +. nf *. 0.2, x0)
    else ctx##lineTo (t0 +. nf *. 0.2, x0)
  in

  ctx##strokeStyle <- Js.string "red";
  ctx##beginPath ();
  Array.iteri plot x_arr;
  ctx##stroke ();

  ctx##strokeStyle <- Js.string "black";
  ctx##beginPath ();
  Array.iteri plot_xaxis x_arr;
  ctx##stroke ();

  ()


(* グラフ描画 *)
let draw_graph t0 x0 x_arr =
  let c = canvas##getContext (Html._2d_) in
  plot_array c t0 x0 x_arr

(* 信号の作成 *)
let create_signal nsmp =
  let x n =
    let t = (float_of_int n) *. h in
    let omega t =
      let k = 28. *. tpi in
      4. *. tpi +. k *. t
    in
    20. *. sin ((omega t) *. t)
  in
  Array.init nsmp (fun n -> x n)


let load_graph () =
  let c = canvas##getContext (Html._2d_) in
  c##clearRect (0., 0., float canvas##width, float canvas##height);
  let x_time = create_signal 1024 in
  draw_graph 10. 100. x_time;
  let freq_in =
    if (!freq < 8) then 8
    else if (8 <= !freq && !freq <= 24) then !freq
    else 24
  in
  let x_wav = Wavelet.gabor x_time 1024. freq_in in
  let x_wav_re = Array.map (fun z -> z.Complex.re *. 0.5) x_wav in
  draw_graph 10. 200. x_wav_re;
  ()
    
let onload _ =
  let main =
    Js.Opt.get (document##getElementById(jstr "main"))
      (fun () -> assert false)
  in
  Dom.appendChild main (int_input "frequency[8-24] : " freq);
  Dom.appendChild main (Html.createBr document);
  Dom.appendChild main
    (button "表示する / display / montrer" (fun _ -> load_graph (); Js._false));
  canvas##width <- 320;
  canvas##height <- 320;
  Dom.appendChild Html.window##document##body canvas;
  load_graph ();
  Js._false


let () =
  Html.window##onload <- Html.handler onload
